// Copyright (c) 1998 Savvy Software
// Created by Matthijs Hollemans
// All Rights Reserved

/**
 * [NOTE] The GUI code is very sloppy, especially the font-sensitivity stuff. 
 *
 * I really wish Be would incorporate font-sensitivity in the Interface Kit.
 * Or maybe I should just learn to use a GUI builder tool;-)
 */
  
// ***************************************************************************
// Implementation System Includes
// ***************************************************************************

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "Main.h"
#include "GrepWindow.h"
#include "Version.h"
#include "Fit.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

/** The messages that its views send to the window. */
enum {
  /** Start the grepping. */
  MSG_START_SEARCH = 1000,

  /** Cancel the grepping. */
  MSG_CANCEL_SEARCH,

  /** The value of the "links" check box has changed. */
  MSG_LINKS_CHECKBOX_TOGGLED,
  
  /** The value of the "subdirectories" check box has changed. */
  MSG_SUBDIR_CHECKBOX_TOGGLED
};

// The initial dimensions of the window.
const float WINDOW_X      = 100;
const float WINDOW_Y      = 100;
const float WINDOW_WIDTH  = 300;
const float WINDOW_HEIGHT = 200;

// The minimum and maximum dimensions of the window.
const float WINDOW_MIN_WIDTH  = WINDOW_WIDTH;
const float WINDOW_MIN_HEIGHT = 150;
const float WINDOW_MAX_WIDTH  = 100000; // This should be large enough.
const float WINDOW_MAX_HEIGHT = 100000; // Anybody got a 40" monitor :-)

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Lifecycle
// ***************************************************************************

// ===========================================================================
// Constructor
// ===========================================================================

tWindow::tWindow() 
  : BWindow(Goowee_Fit(BRect(WINDOW_X, 
                             WINDOW_Y, 
                             WINDOW_X + WINDOW_WIDTH,
                             WINDOW_Y + WINDOW_HEIGHT)),
            "Tracker Grep",
            B_DOCUMENT_WINDOW,
            0)
{
  // Create the background box.
  BBox* lpBox  = new BBox(Bounds(),
          B_EMPTY_STRING, 
          B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP, 
          B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE_JUMP, 
          B_PLAIN_BORDER); 
          
  AddChild(lpBox);

  /**
   * [COMPILER] Specifying a NULL pointer as BMessage gives an error in gcc.
   *
   * My StyleGuideDefs.h defines NULL to be an integer and gcc won't convert
   * this to a BMessage*. Specifying 0 instead of NULL works of course, but 
   * isn't there an other way to do this (apart from casting NULL to 
   * BMessage*) ?
   */  

  // Create the search pattern's entry field. This view does not send a 
  // message when <Enter> is pressed, because the "search/cancel" button
  // already does this (we don't want to send the same message twice.)
  mpSearchText = new BTextControl(BRect(8,
                                        8,
                                        Goowee_Fit(WINDOW_WIDTH)-8,
                                        1),
          "SearchText",
          0,                           // No label.
          0,                           // No initial text.
          0,                           // No message.
          B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP,
          B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE | B_NAVIGABLE); 

  mpSearchText->ResizeToPreferred();   // Because it doesn't have a label.
  lpBox->AddChild(mpSearchText);
  mpSearchText->MakeFocus(TRUE);       // Must come after AddChild().

  // Create the "search/cancel" button
  float lTop = mpSearchText->Frame().bottom+8;
  mpButton = new BButton(BRect(Goowee_Fit(WINDOW_WIDTH-80)-12,
                               lTop,
                               Goowee_Fit(WINDOW_WIDTH)-12,
                               1),
          "Button",
          "Search",
          new BMessage(MSG_START_SEARCH),
          B_FOLLOW_RIGHT);

  mpButton->MakeDefault(TRUE);          
  lpBox->AddChild(mpButton); 
  lpBox->ResizeTo(Goowee_Fit(WINDOW_WIDTH),
                  19 + mpSearchText->Bounds().Height()
                     + mpButton->Bounds().Height());

  // Create the "links" check box.
  lTop += Goowee_Fit(3);
  mpRecurseLinks = new BCheckBox(BRect(8,
                                       lTop,
                                       Goowee_Fit(70),
                                       1),
          "RecurseLinks",
          "Links",
          new BMessage(MSG_LINKS_CHECKBOX_TOGGLED),
          B_FOLLOW_LEFT);
          
  mpRecurseLinks->SetValue(B_CONTROL_OFF);
  lpBox->AddChild(mpRecurseLinks);

  // Create the "subdirectories" check box.
  mpRecurseDirs = new BCheckBox(BRect(Goowee_Fit(80),
                                      lTop,
                                      Goowee_Fit(190),
                                      1),
          "RecurseDirs",
          "Subdirectories",
          new BMessage(MSG_SUBDIR_CHECKBOX_TOGGLED),
          B_FOLLOW_LEFT);
          
  mpRecurseDirs->SetValue(B_CONTROL_ON);
  lpBox->AddChild(mpRecurseDirs);

  // Create the search results' display.
  BRect lRect = Bounds();
  lRect.top = lpBox->Frame().bottom+1;
  mpSearchResults = new BTextView(lRect,
          "SearchResult", 
          BRect(0,0,10000,10000),      // Always enough space to display text.
          B_FOLLOW_ALL_SIDES,
          B_WILL_DRAW);

  mpSearchResults->MakeEditable(FALSE);
  mpSearchResults->MakeSelectable(FALSE);
  mpSearchResults->SetStylable(TRUE);
  mpSearchResults->SetWordWrap(FALSE);
  mpSearchResults->SetAutoindent(FALSE);

  // Write some default text.
  mpSearchResults->SetFontAndColor(be_bold_font);
  mpSearchResults->Insert("Please enter something to search for.");

  // Connect the grepper object to this window.
  gpGrepper->SetTarget(this);
  
  // Put scroll bars around the search results.  
  BScrollView* lpScroller = new BScrollView("ScrollSearchResults", 
          mpSearchResults, 
          B_FOLLOW_ALL_SIDES, 
          B_FULL_UPDATE_ON_RESIZE, 
          TRUE,                        // Horizontal scrollbar.
          TRUE,                        // Vertical scrollbar.
          B_NO_BORDER);
          
  AddChild(lpScroller);
  lpScroller->ResizeTo(Goowee_Fit(WINDOW_WIDTH)+1,
                       lRect.Height()+1);

  // Adjust size limits of the window.  
  SetSizeLimits(Goowee_Fit(WINDOW_MIN_WIDTH), 
                Goowee_Fit(WINDOW_MAX_WIDTH),
                Goowee_Fit(WINDOW_MIN_HEIGHT),
                Goowee_Fit(WINDOW_MAX_HEIGHT));
  Show(); 
}
     
// ***************************************************************************
// PUBLIC Member Function Definitions: Operators
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Operations
// ***************************************************************************

// ===========================================================================
// MessageReceived
// ===========================================================================

void
tWindow::MessageReceived(BMessage* apMessage)
{
  // Figure out what kind of message was received.
  switch(apMessage->what) {
  
    // We need to start a new search. ----------------------------------------
    case MSG_START_SEARCH: {

      // Quit the current grepper thread (if there is one).
      gpGrepper->Stop();
        
      // Clear the search results view.
      mpSearchResults->Delete(0,mpSearchResults->TextLength());
    
      // Only start the search if the user actually entered some text.
      if (0 == strcmp(mpSearchText->Text(),"")) {
        mpSearchResults->SetFontAndColor(be_bold_font);
        mpSearchResults->Insert("Please enter something to search for.");

      // Let's grep.
      } else {
        gpGrepper->SetPattern(mpSearchText->Text());
        if (B_OK == gpGrepper->Start()) {
          mpRecurseDirs->SetEnabled(FALSE);
          mpRecurseLinks->SetEnabled(FALSE);
          mpSearchText->SetEnabled(FALSE);
          mpButton->MakeFocus(TRUE);
          mpButton->SetLabel("Cancel");
          mpButton->SetMessage(new BMessage(MSG_CANCEL_SEARCH));
        }
      }
      break;
    }

    // We need to cancel the search. -----------------------------------------
    case MSG_CANCEL_SEARCH: {
      gpGrepper->Stop();

      // Fall through.
    }
    
    // The grepper object has finished searching. ----------------------------
    case MSG_SEARCH_FINISHED: {
      mpRecurseDirs->SetEnabled(TRUE);
      mpRecurseLinks->SetEnabled(TRUE);
      mpSearchText->SetEnabled(TRUE);
      mpSearchText->MakeFocus(TRUE);
      mpButton->SetLabel("Search");
      mpButton->SetMessage(new BMessage(MSG_START_SEARCH));
      mpSearchText->TextView()->SelectAll();
      break;
    }
    
    // The grepper object sends us a file name. ------------------------------
    case MSG_REPORT_FILE_NAME: {
      tChar lTempString[MAX_LEN_REPORT_TEXT];
      strcpy(lTempString,apMessage->FindString("text"));
      mpSearchResults->SetFontAndColor(be_bold_font);
      mpSearchResults->Insert(lTempString);
      mpSearchResults->SetFontAndColor(be_plain_font);    
      break;
    }
    
    // The grepper object sends us the output of a grep operation. -----------
    case MSG_REPORT_RESULT: {
      tChar lTempString[MAX_LEN_REPORT_TEXT];
      strcpy(lTempString,apMessage->FindString("text"));
      mpSearchResults->SetFontAndColor(be_fixed_font);
      mpSearchResults->Insert(lTempString);
      mpSearchResults->SetFontAndColor(be_plain_font);    
      break;
    }

    // The user clicked on the "links" check box. ----------------------------
    case MSG_LINKS_CHECKBOX_TOGGLED: {
      if (B_CONTROL_ON == mpRecurseLinks->Value()) {
        gpGrepper->SetRecurseLinks(TRUE);
      } else {
        gpGrepper->SetRecurseLinks(FALSE);
      }
      break;
    }
        
    // The user clicked on the "subdirectories" check box. -------------------
    case MSG_SUBDIR_CHECKBOX_TOGGLED: {
      if (B_CONTROL_ON == mpRecurseDirs->Value()) {
        gpGrepper->SetRecurseDirs(TRUE);
      } else {
        gpGrepper->SetRecurseDirs(FALSE);
      }
      break;
    }

    // Let the original BWindow handle the message. --------------------------
    default: {
      BWindow::MessageReceived(apMessage);
      break;
    }
  }
}

// ***************************************************************************
// PUBLIC Member Function Definitions: Access
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Inquiry
// ***************************************************************************

// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************
