// Copyright (c) 1998 Savvy Software
// Created by Matthijs Hollemans
// All Rights Reserved

/**
 * [MAYBE] Remember the settings and search patterns.
 *
 * Every time you change an option, Tracker Grep could save the current 
 * settings (currently only the state of the "links" and "subdirectories" 
 * check boxes) into a file. Likewise, every time you enter a search pattern,
 * Tracker Grep could add it to a file that contains a list of all previous
 * patterns. Instead of entering a new pattern, you can then also pick an old
 * pattern from the search pattern history. 
 */

/**
 * [WARNING] Do not run multiple copies of Tracker Grep at the same time.
 *
 * Starting one copy of Tracker Grep kills _any_ other running grepper 
 * threads (whether from this instance or another), so only one grepper will
 * actually be doing something. Besides that, running multiple copies isn't 
 * tested and will probably crash the Tracker.
 */
 
/**
 * [WARNING] Entering a search pattern that contains two or more separate
 *           words results in funny output and gives unexpected results.
 *
 * Not really a bug, just the way grep works: the first argument is the 
 * search pattern, the rest (i.e. the other words that you entered) are the
 * files to examine. Always put quotes around your search pattern to prevent
 * this from happening. Tracker Grep doesn't do this automatically, allowing
 * you to enter grep options (such as -i) as well.
 */

/**
 * [WARNING] Entering the search pattern /* hangs the machine.
 *
 * Well, not really. Tracker Grep (and the system in general) becomes 
 * unresponsive, because the grepper thread is working like crazy. The same
 * thing happens when you enter "grep /* filename" in the Terminal, so there 
 * is not much I can do about it. There is just one remedy: don't enter funny
 * things like /* !
 */

// ***************************************************************************
// Implementation System Includes
// ***************************************************************************
 
#include <add-ons/tracker/TrackerAddon.h>            // Exports process_refs()

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "StyleGuideDefs.h"          // Standard type and constant definitions
#include "Version.h"
#include "Main.h"
#include "GrepWindow.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

/** Shows the about window. */
static tVoid ShowAboutWindow();

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

tGrepper* gpGrepper;
  
// ***************************************************************************
// PUBLIC Member Function Definitions: Lifecycle
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Operators
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Operations
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Access
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Inquiry
// ***************************************************************************

// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ===========================================================================
// process_refs
// ===========================================================================

// Paraphrased from the Be Book: The Tracker calls this function when the 
// user invokes the add-on. The current directory is found in aDirRef. 
// apMessage is a standard B_REFS_RECEIVED BMessage with the "refs" array 
// containing the entry_refs of the files selected by the user. The third 
// argument is currently unused. 

tVoid 
process_refs(entry_ref aDirRef, BMessage* apMessage, tVoid* apReserved) 
{ 
  // Make the grepper object.
  gpGrepper = new tGrepper(aDirRef, apMessage);
  
  // Create the window. This starts our main loop.
  tWindow* lpWindow = new tWindow();
  
  // Wait until the window thread terminates. If we don't do this, our
  // add-on might crash mysteriously, taking the Tracker with it. See also
  // "Be Newsletter Volume II Issue 10".
  tInt32 lExitValue;
  wait_for_thread(lpWindow->Thread(), &lExitValue);

  // Destroy the grepper object.
  delete gpGrepper;
} 
   
// ===========================================================================
// main
// ===========================================================================

// An add-on is not required to have a main() function. But if it does, 
// the user can run the add-on as a stand-alone application as well. We 
// just show a simple about window.

int
main() 
{
  new BApplication("application/x-vnd.savvy.trackergrep");
  ShowAboutWindow();
  delete be_app;
  return 0;
}

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************

// ===========================================================================
// ShowAboutWindow
// ===========================================================================

/**
 * [COMPILER] The old Metrowerks compiler accepted NULL as an empty string, 
 * but gcc chokes on it. Replacing the NULLs by 0 makes gcc happy again.
 */
 
static tVoid
ShowAboutWindow()
{
  (new BAlert(0,
              "Tracker Grep -- Use grep from the Tracker\n\n"
              "Version " VERSION_STRING "\n"
              "Released on " VERSION_DATE_STRING "\n\n"
              "Copyright " B_UTF8_COPYRIGHT " 1998 Savvy Software\n"
              "Created by Matthijs Hollemans\n"
              "All Rights Reserved\n\n"
              "mahlzeit@bigfoot.com\n"
              "http://home.concepts.nl/~hollies/",
              "Okay",
              0,
              0,
              B_WIDTH_AS_USUAL,
              B_IDEA_ALERT))->Go(); 
}
